/*
 * Narratives for Power BI extension. This file is loaded by the Power BI
 * framework and executed.
 *
 * During development:
 *
 *   1. Go to https://app.powerbi.com/devTools
 *   2. Paste the contents of this file into the TypeScript pane (top left)
 *   3. Click Compile + Run and see the extension load in the preview pane on
 *      the right. Use Chrome DevTools to inspect elements, set breakpoints,
 *      etc.
 */
var powerbi;
(function (powerbi) {
    var visuals;
    (function (visuals) {
        var narrativeAF44D3780B9743AB80246B39523323A3;
        (function (narrativeAF44D3780B9743AB80246B39523323A3) {
            // Modeled after the Orientation module in https://github.com/Microsoft/PowerBI-visuals/blob/master/src/Clients/CustomVisuals/visuals/bulletChart/bulletChart.ts
            var Verbosity;
            (function (Verbosity) {
                Verbosity.type = powerbi.createEnumType([
                    { value: 'low', displayName: 'Low' },
                    { value: 'medium', displayName: 'Medium' },
                    { value: 'high', displayName: 'High' }
                ]);
            })(Verbosity || (Verbosity = {}));
            var Structure;
            (function (Structure) {
                Structure.type = powerbi.createEnumType([
                    { value: 'paragraph', displayName: 'Paragraph' },
                    { value: 'bullets', displayName: 'Bullet List' }
                ]);
            })(Structure || (Structure = {}));
            var ValueType;
            (function (ValueType) {
                ValueType.type = powerbi.createEnumType([
                    { value: 'discrete', displayName: 'Discrete' },
                    { value: 'continuous', displayName: 'Continuous' },
                    { value: 'percentOfWhole', displayName: 'Percent of Whole' },
                    { value: 'scatterplot', displayName: 'Scatterplot' }
                ]);
            })(ValueType || (ValueType = {}));
            var Narrative = (function () {
                function Narrative() {
                }
                // This is called once when the visual is initialially created. It
                // will asynchronously load the extension from our landing page server
                // then instantiate a narrative instance once the script has executed.
                Narrative.prototype.init = function (options) {
                    var _this = this;
                    if (!window.nbi) {
                        $.ajaxSetup({
                            cache: true
                        });
                        var url = Narrative.config.staticUrl + '/main.js' +
                            '?user_key=' + Narrative.config.apiKey +
                            '&code=' + Narrative.config.betaCode;
                        $.getScript(url)
                            .done(function (params) {
                            _this.narrative = window.nbi.initialize(Narrative.config, options);
                            _this.narrative.update(_this.lastUpdateOptions);
                            _this.lastUpdateOptions = null;
                        });
                    }
                    else {
                        this.narrative = window.nbi.initialize(Narrative.config, options);
                    }
                };
                // Update is called for data updates, resizes & formatting changes.
                // Wait until the narrative instance is created before calling its
                // update method.
                Narrative.prototype.update = function (options) {
                    if (!options.dataViews) {
                        return;
                    }
                    this.dataView = options.dataViews[0];
                    if (!this.narrative) {
                        this.lastUpdateOptions = options;
                    }
                    else {
                        this.narrative.update(options);
                    }
                };
                // Method is called when rendering the Power BI formatting pane. It
                // needs to return an array of formatting objects that will be
                // displayed. This function should also take care of handling missing
                // or invalid values. See the capabilities.objects object above.
                Narrative.prototype.enumerateObjectInstances = function (options) {
                    var instances = [];
                    var dataView = this.dataView;
                    switch (options.objectName) {
                        case 'Narrative':
                            var general = {
                                objectName: 'Narrative',
                                displayName: 'Narrative',
                                selector: null,
                                properties: {
                                    Type: Narrative.getValueType(dataView),
                                    Structure: Narrative.getStructure(dataView),
                                    Verbosity: Narrative.getVerbosity(dataView),
                                    'Font Size': Narrative.getFontSize(dataView)
                                }
                            };
                            instances.push(general);
                            break;
                    }
                    return instances;
                };
                // Method will be called when the view mode changes. Interpret the
                // view mode change in the narrative instance.
                Narrative.prototype.onViewModeChanged = function (viewMode) {
                    if (!this.narrative) {
                        return;
                    }
                    this.narrative.setViewMode(viewMode);
                };
                // Get the valueType value from the data view. Defaults to discrete.
                Narrative.getValueType = function (dataView) {
                    var valueType = this.getFormatValue(dataView, 'Type');
                    if (valueType && ~['discrete', 'continuous', 'percentOfWhole', 'scatterplot'].indexOf(valueType)) {
                        return valueType;
                    }
                    return 'discrete';
                };
                // Get the structure value from the data view. Defaults to paragraph.
                Narrative.getStructure = function (dataView) {
                    var structure = this.getFormatValue(dataView, 'Structure');
                    if (structure && ~['paragraph', 'bullets'].indexOf(structure)) {
                        return structure;
                    }
                    return 'paragraph';
                };
                // Get the verbosity value from the data view. Defaults to 5.
                Narrative.getVerbosity = function (dataView) {
                    var verbosity = this.getFormatValue(dataView, 'Verbosity');
                    if (~['low', 'medium', 'high'].indexOf(verbosity)) {
                        return verbosity;
                    }
                    return 'medium';
                };
                // Get the font size value from the data view. Defaults to 14.
                Narrative.getFontSize = function (dataView) {
                    var size = this.getFormatValue(dataView, 'Font Size');
                    if (size) {
                        return size;
                    }
                    return 14;
                };
                // Returns the format value for the given key found in narrative
                // object. This handles missing objects and returns null if nothing
                // was found.
                Narrative.getFormatValue = function (dataView, key) {
                    if (dataView) {
                        var objects = dataView.metadata.objects;
                        if (objects) {
                            var general = objects['Narrative'];
                            if (general) {
                                var value = general[key];
                                if (value != null)
                                    return value;
                            }
                        }
                    }
                    return null;
                };
                Narrative.prototype.destroy = function () {
                    this.narrative.destroy();
                };
                // TODO: add actual formatting options like verbosity, paragraph/
                // bullets as capabilities.
                Narrative.capabilities = {
                    // This is what will appear in the 'Field Wells' in reports
                    dataRoles: [
                        {
                            name: 'Dimensions',
                            displayName: 'Dimensions',
                            kind: powerbi.VisualDataRoleKind.Grouping
                        },
                        {
                            name: 'Values',
                            displayName: 'Values',
                            kind: powerbi.VisualDataRoleKind.Measure
                        },
                    ],
                    // This tells power bi how to map your roles above into the
                    // dataview you will receive
                    dataViewMappings: [{
                            categorical: {
                                categories: {
                                    for: { in: 'Dimensions' },
                                    dataReductionAlgorithm: { top: {} }
                                },
                                values: {
                                    select: [{ bind: { to: 'Values' } }]
                                }
                            }
                        }],
                    // Objects light up the formatting pane
                    objects: {
                        Narrative: {
                            displayName: 'Narrative',
                            properties: {
                                Type: {
                                    displayName: 'Type',
                                    type: { enumeration: ValueType.type }
                                },
                                Structure: {
                                    displayName: 'Structure',
                                    type: { enumeration: Structure.type }
                                },
                                Verbosity: {
                                    displayName: 'Verbosity',
                                    type: { enumeration: Verbosity.type }
                                },
                                'Font Size': {
                                    displayName: 'Font Size',
                                    type: { formatting: { fontSize: true } }
                                }
                            }
                        },
                        // The hidden properties will not be shown (i.e. they won't be
                        // enumerated in enumerateObjectInstances)
                        hidden: {
                            properties: {
                                settings: {
                                    type: { text: true }
                                }
                            }
                        }
                    }
                };
                Narrative.config = {
                    apiKey: '7d6453f8d5eb8e75ee008bc091e0b2b9',
                    apiUrl: 'https://prod-viz-saas-api.narrativescience.com',
                    betaCode: '48f28bd7-2101-44be-b58f-04f9608dfe4b',
                    enableLogger: parseBool('false'),
                    extensionVersion: '0.3.20',
                    mixpanelToken: 'b48ef09750a3a66c7965b94c1403cc16',
                    releaseVersion: '0.0',
                    sentryPublicDsn: 'https://871d577cf2894996909c31577343fddd@app.getsentry.com/69551',
                    staticUrl: 'https://prod-viz-saas-extensions.narrativescience.com/v1/extensions/powerbi/0.0/static',
                    supportUrl: 'https://powerbi-support.narrativescience.com',
                    useCdn: parseBool('true')
                };
                return Narrative;
            }());
            narrativeAF44D3780B9743AB80246B39523323A3.Narrative = Narrative;
        })(narrativeAF44D3780B9743AB80246B39523323A3 = visuals.narrativeAF44D3780B9743AB80246B39523323A3 || (visuals.narrativeAF44D3780B9743AB80246B39523323A3 = {}));
    })(visuals = powerbi.visuals || (powerbi.visuals = {}));
})(powerbi || (powerbi = {}));
function parseBool(str) {
    var strLower = str.toLowerCase();
    if (strLower === 'true' ||
        strLower === '1' ||
        strLower === 'yes') {
        return true;
    }
    return false;
}

var powerbi;
(function (powerbi) {
    var visuals;
    (function (visuals) {
        var plugins;
        (function (plugins) {
            plugins.narrativeAF44D3780B9743AB80246B39523323A3 = {
                name: 'narrativeAF44D3780B9743AB80246B39523323A3',
                class: 'narrativeAF44D3780B9743AB80246B39523323A3',
                capabilities: powerbi.visuals.narrativeAF44D3780B9743AB80246B39523323A3.Narrative.capabilities,
                custom: true,
                create: function () {
                    return new powerbi.visuals.narrativeAF44D3780B9743AB80246B39523323A3.Narrative();
                }
            };
        })(plugins = visuals.plugins || (visuals.plugins = {}));
    })(visuals = powerbi.visuals || (powerbi.visuals = {}));
})(powerbi || (powerbi = {}));
